@@ -14,18 +14,16 @@
 
 #include "test-utils.h"
 
-GMainLoop *loop;
-
 typedef struct {
 	const char *method;
 	const char *path;
 	guint status_code;
 } TestRequest;
 
-struct {
+static struct {
 	TestRequest requests[3];
 } tests[] = {
-	/* A redirecty response to a GET should could a redirect */
+	/* A redirecty response to a GET or HEAD should cause a redirect */
 
 	{ { { "GET", "/301", 301 },
 	    { "GET", "/", 200 },
@@ -39,8 +37,20 @@ struct {
 	{ { { "GET", "/307", 307 },
 	    { "GET", "/", 200 },
 	    { NULL } } },
+	{ { { "HEAD", "/301", 301 },
+	    { "HEAD", "/", 200 },
+	    { NULL } } },
+	{ { { "HEAD", "/302", 302 },
+	    { "HEAD", "/", 200 },
+	    { NULL } } },
+	/* 303 is a nonsensical response to HEAD, so we don't care
+	 * what happens there.
+	 */
+	{ { { "HEAD", "/307", 307 },
+	    { "HEAD", "/", 200 },
+	    { NULL } } },
 
-	/* A non-redirecty response to a GET should not */
+	/* A non-redirecty response to a GET or HEAD should not */
 
 	{ { { "GET", "/300", 300 },
 	    { NULL } } },
@@ -52,18 +62,32 @@ struct {
 	    { NULL } } },
 	{ { { "GET", "/308", 308 },
 	    { NULL } } },
+	{ { { "HEAD", "/300", 300 },
+	    { NULL } } },
+	{ { { "HEAD", "/304", 304 },
+	    { NULL } } },
+	{ { { "HEAD", "/305", 305 },
+	    { NULL } } },
+	{ { { "HEAD", "/306", 306 },
+	    { NULL } } },
+	{ { { "HEAD", "/308", 308 },
+	    { NULL } } },
 	
 	/* Test double-redirect */
 
 	{ { { "GET", "/301/302", 301 },
 	    { "GET", "/302", 302 },
 	    { "GET", "/", 200 } } },
+	{ { { "HEAD", "/301/302", 301 },
+	    { "HEAD", "/302", 302 },
+	    { "HEAD", "/", 200 } } },
 
-	/* POST should only automatically redirect on 303 */
+	/* POST should only automatically redirect on 302 and 303 */
 
 	{ { { "POST", "/301", 301 },
 	    { NULL } } },
 	{ { { "POST", "/302", 302 },
+	    { "GET", "/", 200 },
 	    { NULL } } },
 	{ { { "POST", "/303", 303 },
 	    { "GET", "/", 200 },
@@ -176,8 +200,6 @@ do_redirect_tests (SoupURI *base_uri)
 	g_object_unref (session);
 }
 
-GThread *server_thread;
-
 static void
 server_callback (SoupServer *server, SoupMessage *msg,
 		 const char *path, GHashTable *query,
@@ -187,7 +209,8 @@ server_callback (SoupServer *server, SoupMessage *msg,
 	guint status_code;
 
 	if (!strcmp (path, "/")) {
-		if (msg->method != SOUP_METHOD_GET) {
+		if (msg->method != SOUP_METHOD_GET &&
+		    msg->method != SOUP_METHOD_HEAD) {
 			soup_message_set_status (msg, SOUP_STATUS_METHOD_NOT_ALLOWED);
 			return;
 		}
@@ -198,10 +221,26 @@ server_callback (SoupServer *server, SoupMessage *msg,
 			return;
 		}
 
+		/* Make sure that a HTTP/1.0 redirect doesn't cause an
+		 * HTTP/1.0 re-request. (#521848)
+		 */
+		if (soup_message_get_http_version (msg) == SOUP_HTTP_1_0) {
+			soup_message_set_status (msg, SOUP_STATUS_BAD_REQUEST);
+			return;
+		}
+
 		soup_message_set_status (msg, SOUP_STATUS_OK);
-		soup_message_set_response (msg, "text/plain",
-					   SOUP_MEMORY_STATIC,
-					   "OK\r\n", 4);
+
+		/* FIXME: this is wrong, though it doesn't matter for
+		 * the purposes of this test, and to do the right
+		 * thing currently we'd have to set Content-Length by
+		 * hand.
+		 */
+		if (msg->method != SOUP_METHOD_HEAD) {
+			soup_message_set_response (msg, "text/plain",
+						   SOUP_MEMORY_STATIC,
+						   "OK\r\n", 4);
+		}
 		return;
 	}
 
@@ -212,6 +251,9 @@ server_callback (SoupServer *server, SoupMessage *msg,
 		return;
 	}
 
+	/* See above comment re bug 521848. */
+	soup_message_set_http_version (msg, SOUP_HTTP_1_0);
+
 	soup_message_set_status (msg, status_code);
 	if (*remainder) {
 		soup_message_headers_replace (msg->response_headers,
@@ -222,7 +264,7 @@ server_callback (SoupServer *server, SoupMessage *msg,
 	}
 }
 
-gboolean run_tests = TRUE;
+static gboolean run_tests = TRUE;
 
 static GOptionEntry no_test_entry[] = {
         { "no-tests", 'n', G_OPTION_FLAG_NO_ARG | G_OPTION_FLAG_REVERSE,
